@@ -9,6 +9,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -16,17 +17,21 @@
 
 #include "fio.h"
 #include "server.h"
-#include "crc/crc32.h"
 #include "flist.h"
+#include "hash.h"
 
 struct fio_client {
 	struct flist_head list;
+	struct flist_head hash_list;
 	struct sockaddr_in addr;
+	struct sockaddr_un addr_un;
 	char *hostname;
+	int port;
 	int fd;
 
 	int state;
 	int skip_newline;
+	int is_sock;
 
 	uint16_t argc;
 	char **argv;
@@ -42,15 +47,43 @@ enum {
 
 static FLIST_HEAD(client_list);
 
+#define FIO_CLIENT_HASH_BITS	7
+#define FIO_CLIENT_HASH_SZ	(1 << FIO_CLIENT_HASH_BITS)
+#define FIO_CLIENT_HASH_MASK	(FIO_CLIENT_HASH_SZ - 1)
+static struct flist_head client_hash[FIO_CLIENT_HASH_SZ];
+
 static int handle_client(struct fio_client *client);
 
+static void fio_client_add_hash(struct fio_client *client)
+{
+	int bucket = hash_long(client->fd, FIO_CLIENT_HASH_BITS);
+
+	bucket &= FIO_CLIENT_HASH_MASK;
+	flist_add(&client->hash_list, &client_hash[bucket]);
+}
+
+static void fio_client_remove_hash(struct fio_client *client)
+{
+	if (!flist_empty(&client->hash_list))
+		flist_del_init(&client->hash_list);
+}
+
+static void fio_init fio_client_hash_init(void)
+{
+	int i;
+
+	for (i = 0; i < FIO_CLIENT_HASH_SZ; i++)
+		INIT_FLIST_HEAD(&client_hash[i]);
+}
+
 static struct fio_client *find_client_by_fd(int fd)
 {
+	int bucket = hash_long(fd, FIO_CLIENT_HASH_BITS) & FIO_CLIENT_HASH_MASK;
 	struct fio_client *client;
 	struct flist_head *entry;
 
-	flist_for_each(entry, &client_list) {
-		client = flist_entry(entry, struct fio_client, list);
+	flist_for_each(entry, &client_hash[bucket]) {
+		client = flist_entry(entry, struct fio_client, hash_list);
 
 		if (client->fd == fd)
 			return client;
@@ -59,32 +92,19 @@ static struct fio_client *find_client_by_fd(int fd)
 	return NULL;
 }
 
-static struct fio_client *find_client_by_name(const char *name)
-{
-	struct fio_client *client;
-	struct flist_head *entry;
-
-	flist_for_each(entry, &client_list) {
-		client = flist_entry(entry, struct fio_client, list);
-
-		if (!strcmp(name, client->hostname))
-			return client;
-	}
-
-	return NULL;
-}
-
 static void remove_client(struct fio_client *client)
 {
 	dprint(FD_NET, "client: removed <%s>\n", client->hostname);
 	flist_del(&client->list);
-	nr_clients--;
+
+	fio_client_remove_hash(client);
 
 	free(client->hostname);
 	if (client->argv)
 		free(client->argv);
 
 	free(client);
+	nr_clients--;
 }
 
 static int __fio_client_add_cmd_option(struct fio_client *client,
@@ -105,37 +125,89 @@ static int __fio_client_add_cmd_option(struct fio_client *client,
 	return 0;
 }
 
-int fio_client_add_cmd_option(const char *hostname, const char *opt)
+int fio_client_add_cmd_option(void *cookie, const char *opt)
 {
-	struct fio_client *client;
+	struct fio_client *client = cookie;
 
-	if (!hostname || !opt)
+	if (!client || !opt)
 		return 0;
 
-	client = find_client_by_name(hostname);
-	if (!client) {
-		log_err("fio: unknown client %s\n", hostname);
-		return 1;
-	}
-
 	return __fio_client_add_cmd_option(client, opt);
 }
 
-void fio_client_add(const char *hostname)
+int fio_client_add(const char *hostname, void **cookie)
 {
 	struct fio_client *client;
 
-	dprint(FD_NET, "client: added  <%s>\n", hostname);
 	client = malloc(sizeof(*client));
 	memset(client, 0, sizeof(*client));
 
-	client->hostname = strdup(hostname);
+	INIT_FLIST_HEAD(&client->list);
+	INIT_FLIST_HEAD(&client->hash_list);
+
+	if (fio_server_parse_string(hostname, &client->hostname,
+					&client->is_sock, &client->port,
+					&client->addr.sin_addr))
+		return -1;
+
 	client->fd = -1;
 
 	__fio_client_add_cmd_option(client, "fio");
 
 	flist_add(&client->list, &client_list);
 	nr_clients++;
+	dprint(FD_NET, "client: added <%s>\n", client->hostname);
+	*cookie = client;
+	return 0;
+}
+
+static int fio_client_connect_ip(struct fio_client *client)
+{
+	int fd;
+
+	client->addr.sin_family = AF_INET;
+	client->addr.sin_port = htons(client->port);
+
+	fd = socket(AF_INET, SOCK_STREAM, 0);
+	if (fd < 0) {
+		log_err("fio: socket: %s\n", strerror(errno));
+		return -1;
+	}
+
+	if (connect(fd, (struct sockaddr *) &client->addr, sizeof(client->addr)) < 0) {
+		log_err("fio: connect: %s\n", strerror(errno));
+		log_err("fio: failed to connect to %s\n", client->hostname);
+		close(fd);
+		return -1;
+	}
+
+	return fd;
+}
+
+static int fio_client_connect_sock(struct fio_client *client)
+{
+	struct sockaddr_un *addr = &client->addr_un;
+	fio_socklen_t len;
+	int fd;
+
+	memset(addr, 0, sizeof(*addr));
+	addr->sun_family = AF_UNIX;
+	strcpy(addr->sun_path, client->hostname);
+
+	fd = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (fd < 0) {
+		log_err("fio: socket: %s\n", strerror(errno));
+		return -1;
+	}
+
+	len = sizeof(addr->sun_family) + strlen(addr->sun_path) + 1;
+	if (connect(fd, (struct sockaddr *) addr, len) < 0) {
+		log_err("fio: connect; %s\n", strerror(errno));
+		close(fd);
+		return -1;
+	}
+
+	return fd;
 }
 
 static int fio_client_connect(struct fio_client *client)
@@ -145,36 +217,17 @@ static int fio_client_connect(struct fio_client *client)
 	dprint(FD_NET, "client: connect to host %s\n", client->hostname);
 
 	memset(&client->addr, 0, sizeof(client->addr));
-	client->addr.sin_family = AF_INET;
-	client->addr.sin_port = htons(fio_net_port);
 
-	if (inet_aton(client->hostname, &client->addr.sin_addr) != 1) {
-		struct hostent *hent;
+	if (client->is_sock)
+		fd = fio_client_connect_sock(client);
+	else
+		fd = fio_client_connect_ip(client);
 
-		hent = gethostbyname(client->hostname);
-		if (!hent) {
-			log_err("fio: gethostbyname: %s\n", strerror(errno));
-			log_err("fio: failed looking up hostname %s\n",
-					client->hostname);
-			return 1;
-		}
-
-		memcpy(&client->addr.sin_addr, hent->h_addr, 4);
-	}
-
-	fd = socket(AF_INET, SOCK_STREAM, 0);
-	if (fd < 0) {
-		log_err("fio: socket: %s\n", strerror(errno));
+	if (fd < 0)
 		return 1;
-	}
-
-	if (connect(fd, (struct sockaddr *) &client->addr, sizeof(client->addr)) < 0) {
-		log_err("fio: connect: %s\n", strerror(errno));
-		log_err("fio: failed to connect to %s\n", client->hostname);
-		return 1;
-	}
 
 	client->fd = fd;
+	fio_client_add_hash(client);
 	client->state = Client_connected;
 	return 0;
 }
@@ -195,7 +248,7 @@ void fio_clients_terminate(void)
 
 static void sig_int(int sig)
 {
-	dprint(FD_NET, "client: got sign %d\n", sig);
+	dprint(FD_NET, "client: got signal %d\n", sig);
 	fio_clients_terminate();
 }
 
@@ -288,6 +341,7 @@ static int fio_client_send_ini(struct fio_client *client, const char *filename)
 
 	if (fstat(fd, &sb) < 0) {
 		log_err("fio: job file stat: %s\n", strerror(errno));
+		close(fd);
 		return 1;
 	}
 
@@ -311,11 +365,13 @@ static int fio_client_send_ini(struct fio_client *client, const char *filename)
 
 	if (len) {
 		log_err("fio: failed reading job file %s\n", filename);
+		close(fd);
 		return 1;
 	}
 
 	ret = fio_net_send_cmd(client->fd, FIO_NET_CMD_JOB, buf, sb.st_size);
 	free(buf);
+	close(fd);
 	return ret;
 }
 
@@ -582,7 +638,7 @@ int fio_handle_clients(void)
 
 			client = find_client_by_fd(pfds[i].fd);
 			if (!client) {
-				log_err("fio: unknown client\n");
+				log_err("fio: unknown client fd %d\n", pfds[i].fd);
 				continue;
 			}
 			if (!handle_client(client)) {
