@@ -16,10 +16,6 @@
 
 struct thread_data;
 
-#define FIO_MAJOR	1
-#define FIO_MINOR	99
-#define FIO_PATCH	1
-
 #include "compiler/compiler.h"
 #include "flist.h"
 #include "fifo.h"
