@@ -68,8 +68,11 @@ int log_err(const char *format, ...)
 	if (is_backend)
 		return fio_server_text_output(buffer, len);
 	else {
-		if (f_err != stderr)
-			fwrite(buffer, len, 1, stderr);
+		if (f_err != stderr) {
+			int fio_unused ret;
+
+			ret = fwrite(buffer, len, 1, stderr);
+		}
 
 		return fwrite(buffer, len, 1, f_err);
 	}
