@@ -26,11 +26,10 @@ struct fio_net_cmd {
 };
 
 enum {
-	FIO_SERVER_VER		= 2,
-	FIO_SERVER_VER1		= 1,
-	FIO_SERVER_VER2		= 2,
+	FIO_SERVER_VER		= 3,
+	FIO_SERVER_VER3		= 3,
 
-	FIO_SERVER_MAX_PDU	= 64,
+	FIO_SERVER_MAX_PDU	= 1024,
 
 	FIO_NET_CMD_QUIT	= 1,
 	FIO_NET_CMD_EXIT	= 2,
@@ -76,6 +75,8 @@ extern int fio_server_text_output(const char *, unsigned int len);
 extern int fio_server_log(const char *format, ...);
 extern int fio_net_send_cmd(int, uint16_t, const void *, off_t);
 extern int fio_net_send_simple_cmd(int sk, uint16_t opcode, uint64_t serial);
+extern void fio_server_set_arg(const char *);
+extern int fio_server_parse_string(const char *, char **, int *, int *, struct in_addr *);
 
 struct thread_stat;
 struct group_run_stats;
@@ -87,8 +88,8 @@ extern void fio_server_idle_loop(void);
 extern int fio_clients_connect(void);
 extern int fio_clients_send_ini(const char *);
 extern int fio_handle_clients(void);
-extern void fio_client_add(const char *);
-extern int fio_client_add_cmd_option(const char *, const char *);
+extern int fio_client_add(const char *, void **);
+extern int fio_client_add_cmd_option(void *, const char *);
 
 extern int fio_recv_data(int sk, void *p, unsigned int len);
 extern int fio_send_data(int sk, const void *p, unsigned int len);
@@ -103,7 +104,7 @@ static inline void fio_init_net_cmd(struct fio_net_cmd *cmd, uint16_t opcode,
 {
 	memset(cmd, 0, sizeof(*cmd));
 
-	cmd->version	= __cpu_to_le16(FIO_SERVER_VER2);
+	cmd->version	= __cpu_to_le16(FIO_SERVER_VER3);
 	cmd->opcode	= cpu_to_le16(opcode);
 
 	if (pdu) {
